@@ -1,10 +1,14 @@
 mod array;
+mod cell;
+mod cstring;
 mod ipaddr;
 mod map;
-mod num;
+pub(crate) mod num;
+mod option;
 mod osstr;
 mod path;
 mod set;
 mod slice;
 mod string;
+mod time;
 mod vec;
