@@ -18,7 +18,7 @@ fn foo(
 fn my_module(m: &crate::Bound<'_, crate::types::PyModule>) -> crate::PyResult<()> {
     <crate::Bound<'_, crate::types::PyModule> as crate::types::PyModuleMethods>::add_function(
         m,
-        crate::wrap_pyfunction_bound!(do_something, m)?,
+        crate::wrap_pyfunction!(do_something, m)?,
     )?;
     <crate::Bound<'_, crate::types::PyModule> as crate::types::PyModuleMethods>::add_wrapped(
         m,
@@ -33,4 +33,7 @@ fn my_module(m: &crate::Bound<'_, crate::types::PyModule>) -> crate::PyResult<()
 mod my_module_declarative {
     #[pymodule_export]
     use super::{do_something, foo};
+
+    #[pymodule_export]
+    const BAR: u32 = 42;
 }
