@@ -1,6 +1,6 @@
 #![cfg(feature = "anyhow")]
 
-use pyo3::wrap_pyfunction_bound;
+use pyo3::{ffi, wrap_pyfunction};
 
 #[test]
 fn test_anyhow_py_function_ok_result() {
@@ -12,8 +12,8 @@ fn test_anyhow_py_function_ok_result() {
         Ok(String::from("OK buddy"))
     }
 
-    Python::with_gil(|py| {
-        let func = wrap_pyfunction_bound!(produce_ok_result)(py).unwrap();
+    Python::attach(|py| {
+        let func = wrap_pyfunction!(produce_ok_result)(py).unwrap();
 
         py_run!(
             py,
@@ -35,15 +35,17 @@ fn test_anyhow_py_function_err_result() {
         anyhow::bail!("error time")
     }
 
-    Python::with_gil(|py| {
-        let func = wrap_pyfunction_bound!(produce_err_result)(py).unwrap();
-        let locals = PyDict::new_bound(py);
+    Python::attach(|py| {
+        let func = wrap_pyfunction!(produce_err_result)(py).unwrap();
+        let locals = PyDict::new(py);
         locals.set_item("func", func).unwrap();
 
-        py.run_bound(
-            r#"
+        py.run(
+            ffi::c_str!(
+                r#"
             func()
-            "#,
+            "#
+            ),
             None,
             Some(&locals),
         )
