@@ -1,12 +1,18 @@
 #![cfg(feature = "macros")]
 
+#[cfg(not(Py_LIMITED_API))]
+use pyo3::exceptions::PyWarning;
+use pyo3::exceptions::{PyFutureWarning, PyUserWarning};
 use pyo3::prelude::*;
 use pyo3::py_run;
+use pyo3::types::PySequence;
 use pyo3::types::{IntoPyDict, PyDict, PyList, PySet, PyString, PyTuple, PyType};
-use pyo3::PyCell;
+use pyo3::BoundObject;
+use pyo3_macros::pyclass;
 
-#[path = "../src/tests/common.rs"]
-mod common;
+use crate::test_utils::CatchWarnings;
+
+mod test_utils;
 
 #[pyclass]
 struct InstanceMethod {
@@ -28,8 +34,8 @@ impl InstanceMethod {
 
 #[test]
 fn instance_method() {
-    Python::with_gil(|py| {
-        let obj = PyCell::new(py, InstanceMethod { member: 42 }).unwrap();
+    Python::attach(|py| {
+        let obj = Bound::new(py, InstanceMethod { member: 42 }).unwrap();
         let obj_ref = obj.borrow();
         assert_eq!(obj_ref.method(), 42);
         py_assert!(py, obj, "obj.method() == 42");
@@ -52,8 +58,8 @@ impl InstanceMethodWithArgs {
 
 #[test]
 fn instance_method_with_args() {
-    Python::with_gil(|py| {
-        let obj = PyCell::new(py, InstanceMethodWithArgs { member: 7 }).unwrap();
+    Python::attach(|py| {
+        let obj = Bound::new(py, InstanceMethodWithArgs { member: 7 }).unwrap();
         let obj_ref = obj.borrow();
         assert_eq!(obj_ref.method(6), 42);
         py_assert!(py, obj, "obj.method(3) == 21");
@@ -73,23 +79,22 @@ impl ClassMethod {
 
     #[classmethod]
     /// Test class method.
-    fn method(cls: &PyType) -> PyResult<String> {
-        Ok(format!("{}.method()!", cls.name()?))
+    fn method(cls: &Bound<'_, PyType>) -> PyResult<String> {
+        Ok(format!("{}.method()!", cls.qualname()?))
     }
 
     #[classmethod]
-    fn method_owned(cls: Py<PyType>) -> PyResult<String> {
-        Ok(format!(
-            "{}.method_owned()!",
-            Python::with_gil(|gil| cls.as_ref(gil).name().map(ToString::to_string))?
-        ))
+    fn method_owned(cls: Py<PyType>, py: Python<'_>) -> PyResult<String> {
+        Ok(format!("{}.method_owned()!", cls.bind(py).qualname()?))
     }
 }
 
 #[test]
 fn class_method() {
-    Python::with_gil(|py| {
-        let d = [("C", py.get_type::<ClassMethod>())].into_py_dict(py);
+    Python::attach(|py| {
+        let d = [("C", py.get_type::<ClassMethod>())]
+            .into_py_dict(py)
+            .unwrap();
         py_assert!(py, *d, "C.method() == 'ClassMethod.method()!'");
         py_assert!(py, *d, "C().method() == 'ClassMethod.method()!'");
         py_assert!(
@@ -108,15 +113,17 @@ struct ClassMethodWithArgs {}
 #[pymethods]
 impl ClassMethodWithArgs {
     #[classmethod]
-    fn method(cls: &PyType, input: &PyString) -> PyResult<String> {
-        Ok(format!("{}.method({})", cls.name()?, input))
+    fn method(cls: &Bound<'_, PyType>, input: &Bound<'_, PyString>) -> PyResult<String> {
+        Ok(format!("{}.method({})", cls.qualname()?, input))
     }
 }
 
 #[test]
 fn class_method_with_args() {
-    Python::with_gil(|py| {
-        let d = [("C", py.get_type::<ClassMethodWithArgs>())].into_py_dict(py);
+    Python::attach(|py| {
+        let d = [("C", py.get_type::<ClassMethodWithArgs>())]
+            .into_py_dict(py)
+            .unwrap();
         py_assert!(
             py,
             *d,
@@ -144,10 +151,12 @@ impl StaticMethod {
 
 #[test]
 fn static_method() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         assert_eq!(StaticMethod::method(py), "StaticMethod.method()!");
 
-        let d = [("C", py.get_type::<StaticMethod>())].into_py_dict(py);
+        let d = [("C", py.get_type::<StaticMethod>())]
+            .into_py_dict(py)
+            .unwrap();
         py_assert!(py, *d, "C.method() == 'StaticMethod.method()!'");
         py_assert!(py, *d, "C().method() == 'StaticMethod.method()!'");
         py_assert!(py, *d, "C.method.__doc__ == 'Test static method.'");
@@ -162,16 +171,18 @@ struct StaticMethodWithArgs {}
 impl StaticMethodWithArgs {
     #[staticmethod]
     fn method(_py: Python<'_>, input: i32) -> String {
-        format!("0x{:x}", input)
+        format!("0x{input:x}")
     }
 }
 
 #[test]
 fn static_method_with_args() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         assert_eq!(StaticMethodWithArgs::method(py, 1234), "0x4d2");
 
-        let d = [("C", py.get_type::<StaticMethodWithArgs>())].into_py_dict(py);
+        let d = [("C", py.get_type::<StaticMethodWithArgs>())]
+            .into_py_dict(py)
+            .unwrap();
         py_assert!(py, *d, "C.method(1337) == '0x539'");
     });
 }
@@ -189,6 +200,7 @@ impl MethSignature {
     fn get_optional2(&self, test: Option<i32>) -> Option<i32> {
         test
     }
+    #[pyo3(signature=(_t1 = None, t2 = None, _t3 = None))]
     fn get_optional_positional(
         &self,
         _t1: Option<i32>,
@@ -207,19 +219,33 @@ impl MethSignature {
         test
     }
     #[pyo3(signature = (*args, **kwargs))]
-    fn get_kwargs(&self, py: Python<'_>, args: &PyTuple, kwargs: Option<&PyDict>) -> PyObject {
-        [args.into(), kwargs.to_object(py)].to_object(py)
+    fn get_kwargs<'py>(
+        &self,
+        py: Python<'py>,
+        args: &Bound<'py, PyTuple>,
+        kwargs: Option<&Bound<'py, PyDict>>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        [
+            args.as_any().clone(),
+            kwargs.into_pyobject(py)?.into_any().into_bound(),
+        ]
+        .into_pyobject(py)
     }
 
     #[pyo3(signature = (a, *args, **kwargs))]
-    fn get_pos_arg_kw(
+    fn get_pos_arg_kw<'py>(
         &self,
-        py: Python<'_>,
+        py: Python<'py>,
         a: i32,
-        args: &PyTuple,
-        kwargs: Option<&PyDict>,
-    ) -> PyObject {
-        [a.to_object(py), args.into(), kwargs.to_object(py)].to_object(py)
+        args: &Bound<'py, PyTuple>,
+        kwargs: Option<&Bound<'py, PyDict>>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        [
+            a.into_pyobject(py)?.into_any().into_bound(),
+            args.as_any().clone(),
+            kwargs.into_pyobject(py)?.into_any().into_bound(),
+        ]
+        .into_pyobject(py)
     }
 
     #[pyo3(signature = (a, b, /))]
@@ -262,9 +288,14 @@ impl MethSignature {
         &self,
         py: Python<'_>,
         a: i32,
-        kwargs: Option<&PyDict>,
-    ) -> PyObject {
-        [a.to_object(py), kwargs.to_object(py)].to_object(py)
+        kwargs: Option<&Bound<'_, PyDict>>,
+    ) -> PyResult<Py<PyAny>> {
+        [
+            a.into_pyobject(py)?.into_any().into_bound(),
+            kwargs.into_pyobject(py)?.into_any().into_bound(),
+        ]
+        .into_pyobject(py)
+        .map(Bound::unbind)
     }
 
     #[pyo3(signature = (a=0, /, **kwargs))]
@@ -272,9 +303,14 @@ impl MethSignature {
         &self,
         py: Python<'_>,
         a: i32,
-        kwargs: Option<&PyDict>,
-    ) -> PyObject {
-        [a.to_object(py), kwargs.to_object(py)].to_object(py)
+        kwargs: Option<&Bound<'_, PyDict>>,
+    ) -> PyResult<Py<PyAny>> {
+        [
+            a.into_pyobject(py)?.into_any().into_bound(),
+            kwargs.into_pyobject(py)?.into_any().into_bound(),
+        ]
+        .into_pyobject(py)
+        .map(Bound::unbind)
     }
 
     #[pyo3(signature = (*, a = 2, b = 3))]
@@ -293,8 +329,16 @@ impl MethSignature {
     }
 
     #[pyo3(signature = (*args, a))]
-    fn get_args_and_required_keyword(&self, py: Python<'_>, args: &PyTuple, a: i32) -> PyObject {
-        (args, a).to_object(py)
+    fn get_args_and_required_keyword(
+        &self,
+        py: Python<'_>,
+        args: &Bound<'_, PyTuple>,
+        a: i32,
+    ) -> PyResult<Py<PyAny>> {
+        (args, a)
+            .into_pyobject(py)
+            .map(BoundObject::into_any)
+            .map(Bound::unbind)
     }
 
     #[pyo3(signature = (a, b = 2, *, c = 3))]
@@ -308,8 +352,18 @@ impl MethSignature {
     }
 
     #[pyo3(signature = (a, **kwargs))]
-    fn get_pos_kw(&self, py: Python<'_>, a: i32, kwargs: Option<&PyDict>) -> PyObject {
-        [a.to_object(py), kwargs.to_object(py)].to_object(py)
+    fn get_pos_kw(
+        &self,
+        py: Python<'_>,
+        a: i32,
+        kwargs: Option<&Bound<'_, PyDict>>,
+    ) -> PyResult<Py<PyAny>> {
+        [
+            a.into_pyobject(py)?.into_any().into_bound(),
+            kwargs.into_pyobject(py)?.into_any().into_bound(),
+        ]
+        .into_pyobject(py)
+        .map(Bound::unbind)
     }
 
     // "args" can be anything that can be extracted from PyTuple
@@ -321,7 +375,7 @@ impl MethSignature {
 
 #[test]
 fn meth_signature() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let inst = Py::new(py, MethSignature {}).unwrap();
 
         py_run!(py, inst, "assert inst.get_optional() == 10");
@@ -668,8 +722,8 @@ impl MethDocs {
 
 #[test]
 fn meth_doc() {
-    Python::with_gil(|py| {
-        let d = [("C", py.get_type::<MethDocs>())].into_py_dict(py);
+    Python::attach(|py| {
+        let d = [("C", py.get_type::<MethDocs>())].into_py_dict(py).unwrap();
         py_assert!(py, *d, "C.__doc__ == 'A class with \"documentation\".'");
         py_assert!(
             py,
@@ -689,12 +743,13 @@ struct MethodWithLifeTime {}
 
 #[pymethods]
 impl MethodWithLifeTime {
-    fn set_to_list<'py>(&self, py: Python<'py>, set: &'py PySet) -> PyResult<&'py PyList> {
+    fn set_to_list<'py>(&self, set: &Bound<'py, PySet>) -> PyResult<Bound<'py, PyList>> {
+        let py = set.py();
         let mut items = vec![];
         for _ in 0..set.len() {
             items.push(set.pop().unwrap());
         }
-        let list = PyList::new(py, items);
+        let list = PyList::new(py, items)?;
         list.sort()?;
         Ok(list)
     }
@@ -702,8 +757,8 @@ impl MethodWithLifeTime {
 
 #[test]
 fn method_with_lifetime() {
-    Python::with_gil(|py| {
-        let obj = PyCell::new(py, MethodWithLifeTime {}).unwrap();
+    Python::attach(|py| {
+        let obj = Py::new(py, MethodWithLifeTime {}).unwrap();
         py_run!(
             py,
             obj,
@@ -736,11 +791,13 @@ impl MethodWithPyClassArg {
     fn inplace_add_pyref(&self, mut other: PyRefMut<'_, MethodWithPyClassArg>) {
         other.value += self.value;
     }
+    #[pyo3(signature=(other = None))]
     fn optional_add(&self, other: Option<&MethodWithPyClassArg>) -> MethodWithPyClassArg {
         MethodWithPyClassArg {
             value: self.value + other.map(|o| o.value).unwrap_or(10),
         }
     }
+    #[pyo3(signature=(other = None))]
     fn optional_inplace_add(&self, other: Option<&mut MethodWithPyClassArg>) {
         if let Some(other) = other {
             other.value += self.value;
@@ -750,10 +807,10 @@ impl MethodWithPyClassArg {
 
 #[test]
 fn method_with_pyclassarg() {
-    Python::with_gil(|py| {
-        let obj1 = PyCell::new(py, MethodWithPyClassArg { value: 10 }).unwrap();
-        let obj2 = PyCell::new(py, MethodWithPyClassArg { value: 10 }).unwrap();
-        let d = [("obj1", obj1), ("obj2", obj2)].into_py_dict(py);
+    Python::attach(|py| {
+        let obj1 = Py::new(py, MethodWithPyClassArg { value: 10 }).unwrap();
+        let obj2 = Py::new(py, MethodWithPyClassArg { value: 10 }).unwrap();
+        let d = [("obj1", obj1), ("obj2", obj2)].into_py_dict(py).unwrap();
         py_run!(py, *d, "obj = obj1.add(obj2); assert obj.value == 20");
         py_run!(py, *d, "obj = obj1.add_pyref(obj2); assert obj.value == 20");
         py_run!(py, *d, "obj = obj1.optional_add(); assert obj.value == 20");
@@ -813,7 +870,7 @@ impl CfgStruct {
 
 #[test]
 fn test_cfg_attrs() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let inst = Py::new(py, CfgStruct {}).unwrap();
 
         #[cfg(unix)]
@@ -842,10 +899,11 @@ struct FromSequence {
 #[pymethods]
 impl FromSequence {
     #[new]
-    fn new(seq: Option<&pyo3::types::PySequence>) -> PyResult<Self> {
+    #[pyo3(signature=(seq = None))]
+    fn new(seq: Option<&Bound<'_, PySequence>>) -> PyResult<Self> {
         if let Some(seq) = seq {
             Ok(FromSequence {
-                numbers: seq.as_ref().extract::<Vec<_>>()?,
+                numbers: seq.as_any().extract::<Vec<_>>()?,
             })
         } else {
             Ok(FromSequence::default())
@@ -855,7 +913,7 @@ impl FromSequence {
 
 #[test]
 fn test_from_sequence() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let typeobj = py.get_type::<FromSequence>();
         py_assert!(py, typeobj, "typeobj(range(0, 4)).numbers == [0, 1, 2, 3]");
     });
@@ -864,9 +922,9 @@ fn test_from_sequence() {
 #[pyclass]
 struct r#RawIdents {
     #[pyo3(get, set)]
-    r#type: PyObject,
-    r#subtype: PyObject,
-    r#subsubtype: PyObject,
+    r#type: Py<PyAny>,
+    r#subtype: Py<PyAny>,
+    r#subsubtype: Py<PyAny>,
 }
 
 #[pymethods]
@@ -874,9 +932,9 @@ impl r#RawIdents {
     #[new]
     pub fn r#new(
         r#_py: Python<'_>,
-        r#type: PyObject,
-        r#subtype: PyObject,
-        r#subsubtype: PyObject,
+        r#type: Py<PyAny>,
+        r#subtype: Py<PyAny>,
+        r#subsubtype: Py<PyAny>,
     ) -> Self {
         Self {
             r#type,
@@ -886,36 +944,36 @@ impl r#RawIdents {
     }
 
     #[getter(r#subtype)]
-    pub fn r#get_subtype(&self) -> PyObject {
-        self.r#subtype.clone()
+    pub fn r#get_subtype(&self, py: Python<'_>) -> Py<PyAny> {
+        self.r#subtype.clone_ref(py)
     }
 
     #[setter(r#subtype)]
-    pub fn r#set_subtype(&mut self, r#subtype: PyObject) {
+    pub fn r#set_subtype(&mut self, r#subtype: Py<PyAny>) {
         self.r#subtype = r#subtype;
     }
 
     #[getter]
-    pub fn r#get_subsubtype(&self) -> PyObject {
-        self.r#subsubtype.clone()
+    pub fn r#get_subsubtype(&self, py: Python<'_>) -> Py<PyAny> {
+        self.r#subsubtype.clone_ref(py)
     }
 
     #[setter]
-    pub fn r#set_subsubtype(&mut self, r#subsubtype: PyObject) {
+    pub fn r#set_subsubtype(&mut self, r#subsubtype: Py<PyAny>) {
         self.r#subsubtype = r#subsubtype;
     }
 
-    pub fn r#__call__(&mut self, r#type: PyObject) {
+    pub fn r#__call__(&mut self, r#type: Py<PyAny>) {
         self.r#type = r#type;
     }
 
     #[staticmethod]
-    pub fn r#static_method(r#type: PyObject) -> PyObject {
+    pub fn r#static_method(r#type: Py<PyAny>) -> Py<PyAny> {
         r#type
     }
 
     #[classmethod]
-    pub fn r#class_method(_: &PyType, r#type: PyObject) -> PyObject {
+    pub fn r#class_method(_: &Bound<'_, PyType>, r#type: Py<PyAny>) -> Py<PyAny> {
         r#type
     }
 
@@ -935,9 +993,9 @@ impl r#RawIdents {
 
 #[test]
 fn test_raw_idents() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let raw_idents_type = py.get_type::<r#RawIdents>();
-        assert_eq!(raw_idents_type.name().unwrap(), "RawIdents");
+        assert_eq!(raw_idents_type.qualname().unwrap(), "RawIdents");
         py_run!(
             py,
             raw_idents_type,
@@ -1017,48 +1075,53 @@ macro_rules! issue_1506 {
 issue_1506!(
     #[pymethods]
     impl Issue1506 {
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506(
             &self,
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
 
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_mut(
             &mut self,
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
 
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_custom_receiver(
             _slf: Py<Self>,
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
 
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_custom_receiver_explicit(
             _slf: Py<Issue1506>,
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
 
         #[new]
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_new(
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) -> Self {
             Issue1506 {}
         }
@@ -1072,21 +1135,23 @@ issue_1506!(
         fn issue_1506_setter(&self, _py: Python<'_>, _value: i32) {}
 
         #[staticmethod]
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_static(
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
 
         #[classmethod]
+        #[pyo3(signature = (_arg, _args, _kwargs=None))]
         fn issue_1506_class(
-            _cls: &PyType,
+            _cls: &Bound<'_, PyType>,
             _py: Python<'_>,
-            _arg: &PyAny,
-            _args: &PyTuple,
-            _kwargs: Option<&PyDict>,
+            _arg: &Bound<'_, PyAny>,
+            _args: &Bound<'_, PyTuple>,
+            _kwargs: Option<&Bound<'_, PyDict>>,
         ) {
         }
     }
@@ -1115,7 +1180,7 @@ fn test_option_pyclass_arg() {
         arg.map(|_| SomePyClass {})
     }
 
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let f = wrap_pyfunction!(option_class_arg, py).unwrap();
         assert!(f.call0().unwrap().is_none());
         let obj = Py::new(py, SomePyClass {}).unwrap();
@@ -1138,7 +1203,270 @@ fn test_issue_2988() {
         _data: Vec<i32>,
         // The from_py_with here looks a little odd, we just need some way
         // to encourage the macro to expand the from_py_with default path too
-        #[pyo3(from_py_with = "PyAny::extract")] _data2: Vec<i32>,
+        #[pyo3(from_py_with = <Bound<'_, _> as PyAnyMethods>::extract)] _data2: Vec<i32>,
     ) {
     }
 }
+
+#[cfg(not(Py_LIMITED_API))]
+#[pyclass(extends=PyWarning)]
+pub struct UserDefinedWarning {}
+
+#[cfg(not(Py_LIMITED_API))]
+#[pymethods]
+impl UserDefinedWarning {
+    #[new]
+    #[pyo3(signature = (*_args, **_kwargs))]
+    fn new(_args: Bound<'_, PyAny>, _kwargs: Option<Bound<'_, PyAny>>) -> Self {
+        Self {}
+    }
+}
+
+#[test]
+fn test_pymethods_warn() {
+    // We do not test #[classattr] nor __traverse__
+    // because it doesn't make sense to implement deprecated methods for them.
+
+    #[pyclass]
+    struct WarningMethodContainer {
+        value: i32,
+    }
+
+    #[pymethods]
+    impl WarningMethodContainer {
+        #[new]
+        #[pyo3(warn(message = "this __new__ method raises warning"))]
+        fn new() -> Self {
+            Self { value: 0 }
+        }
+
+        #[pyo3(warn(message = "this method raises warning"))]
+        fn method_with_warning(_slf: PyRef<'_, Self>) {}
+
+        #[pyo3(warn(message = "this method raises warning", category = PyFutureWarning))]
+        fn method_with_warning_and_custom_category(_slf: PyRef<'_, Self>) {}
+
+        #[cfg(not(Py_LIMITED_API))]
+        #[pyo3(warn(message = "this method raises user-defined warning", category = UserDefinedWarning))]
+        fn method_with_warning_and_user_defined_category(&self) {}
+
+        #[staticmethod]
+        #[pyo3(warn(message = "this static method raises warning"))]
+        fn static_method() {}
+
+        #[staticmethod]
+        #[pyo3(warn(message = "this class method raises warning"))]
+        fn class_method() {}
+
+        #[getter]
+        #[pyo3(warn(message = "this getter raises warning"))]
+        fn get_value(&self) -> i32 {
+            self.value
+        }
+
+        #[setter]
+        #[pyo3(warn(message = "this setter raises warning"))]
+        fn set_value(&mut self, value: i32) {
+            self.value = value;
+        }
+
+        #[pyo3(warn(message = "this subscript op method raises warning"))]
+        fn __getitem__(&self, _key: i32) -> i32 {
+            0
+        }
+
+        #[pyo3(warn(message = "the + op method raises warning"))]
+        fn __add__(&self, other: PyRef<'_, Self>) -> Self {
+            Self {
+                value: self.value + other.value,
+            }
+        }
+
+        #[pyo3(warn(message = "this __call__ method raises warning"))]
+        fn __call__(&self) -> i32 {
+            self.value
+        }
+    }
+
+    Python::attach(|py| {
+        let typeobj = py.get_type::<WarningMethodContainer>();
+        let obj = CatchWarnings::enter(py, |_| typeobj.call0()).unwrap();
+
+        // FnType::Fn
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.method_with_warning()",
+            [("this method raises warning", PyUserWarning)],
+        );
+
+        // FnType::Fn
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.method_with_warning_and_custom_category()",
+            [("this method raises warning", PyFutureWarning)]
+        );
+
+        // FnType::Fn, user-defined warning
+        #[cfg(not(Py_LIMITED_API))]
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.method_with_warning_and_user_defined_category()",
+            [(
+                "this method raises user-defined warning",
+                UserDefinedWarning
+            )]
+        );
+
+        // #[staticmethod], FnType::FnStatic
+        py_expect_warning!(
+            py,
+            typeobj,
+            "typeobj.static_method()",
+            [("this static method raises warning", PyUserWarning)]
+        );
+
+        // #[classmethod], FnType::FnClass
+        py_expect_warning!(
+            py,
+            typeobj,
+            "typeobj.class_method()",
+            [("this class method raises warning", PyUserWarning)]
+        );
+
+        // #[classmethod], FnType::FnClass
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.class_method()",
+            [("this class method raises warning", PyUserWarning)]
+        );
+
+        // #[new], FnType::FnNew
+        py_expect_warning!(
+            py,
+            typeobj,
+            "typeobj()",
+            [("this __new__ method raises warning", PyUserWarning)]
+        );
+
+        // #[getter], FnType::Getter
+        py_expect_warning!(
+            py,
+            obj,
+            "val = obj.value",
+            [("this getter raises warning", PyUserWarning)]
+        );
+
+        // #[setter], FnType::Setter
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.value = 10",
+            [("this setter raises warning", PyUserWarning)]
+        );
+
+        // PyMethodProtoKind::Slot
+        py_expect_warning!(
+            py,
+            obj,
+            "obj[0]",
+            [("this subscript op method raises warning", PyUserWarning)]
+        );
+
+        // PyMethodProtoKind::SlotFragment
+        py_expect_warning!(
+            py,
+            obj,
+            "obj + obj",
+            [("the + op method raises warning", PyUserWarning)]
+        );
+
+        // PyMethodProtoKind::Call
+        py_expect_warning!(
+            py,
+            obj,
+            "obj()",
+            [("this __call__ method raises warning", PyUserWarning)]
+        );
+    });
+
+    #[pyclass]
+    struct WarningMethodContainer2 {}
+
+    #[pymethods]
+    impl WarningMethodContainer2 {
+        #[new]
+        #[classmethod]
+        #[pyo3(warn(message = "this class-method __new__ method raises warning"))]
+        fn new(_cls: Bound<'_, PyType>) -> Self {
+            Self {}
+        }
+    }
+
+    Python::attach(|py| {
+        let typeobj = py.get_type::<WarningMethodContainer2>();
+
+        // #[new], #[classmethod], FnType::FnNewClass
+        py_expect_warning!(
+            py,
+            typeobj,
+            "typeobj()",
+            [(
+                "this class-method __new__ method raises warning",
+                PyUserWarning
+            )]
+        );
+    });
+}
+
+#[test]
+fn test_py_methods_multiple_warn() {
+    #[pyclass]
+    struct MultipleWarnContainer {}
+
+    #[pymethods]
+    impl MultipleWarnContainer {
+        #[new]
+        fn new() -> Self {
+            Self {}
+        }
+
+        #[pyo3(warn(message = "this method raises warning 1"))]
+        #[pyo3(warn(message = "this method raises warning 2", category = PyFutureWarning))]
+        fn multiple_warn_method(&self) {}
+
+        #[cfg(not(Py_LIMITED_API))]
+        #[pyo3(warn(message = "this method raises FutureWarning", category = PyFutureWarning))]
+        #[pyo3(warn(message = "this method raises UserDefinedWarning", category = UserDefinedWarning))]
+        fn multiple_warn_custom_category_method(&self) {}
+    }
+
+    Python::attach(|py| {
+        let typeobj = py.get_type::<MultipleWarnContainer>();
+        let obj = typeobj.call0().unwrap();
+
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.multiple_warn_method()",
+            [
+                ("this method raises warning 1", PyUserWarning),
+                ("this method raises warning 2", PyFutureWarning)
+            ]
+        );
+
+        #[cfg(not(Py_LIMITED_API))]
+        py_expect_warning!(
+            py,
+            obj,
+            "obj.multiple_warn_custom_category_method()",
+            [
+                ("this method raises FutureWarning", PyFutureWarning),
+                ("this method raises UserDefinedWarning", UserDefinedWarning)
+            ]
+        );
+    });
+}
