@@ -1,9 +1,13 @@
 #![cfg(feature = "macros")]
+#![warn(unsafe_op_in_unsafe_fn)]
 
 use std::collections::HashMap;
 
 #[cfg(not(Py_LIMITED_API))]
 use pyo3::buffer::PyBuffer;
+#[cfg(not(Py_LIMITED_API))]
+use pyo3::exceptions::PyWarning;
+use pyo3::exceptions::{PyFutureWarning, PyUserWarning};
 use pyo3::ffi::c_str;
 use pyo3::prelude::*;
 #[cfg(not(Py_LIMITED_API))]
@@ -11,17 +15,17 @@ use pyo3::types::PyDateTime;
 #[cfg(not(any(Py_LIMITED_API, PyPy)))]
 use pyo3::types::PyFunction;
 use pyo3::types::{self, PyCFunction};
+use pyo3_macros::pyclass;
 
-#[path = "../src/tests/common.rs"]
-mod common;
+mod test_utils;
 
 #[pyfunction(name = "struct")]
 fn struct_function() {}
 
 #[test]
 fn test_rust_keyword_name() {
-    Python::with_gil(|py| {
-        let f = wrap_pyfunction_bound!(struct_function)(py).unwrap();
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(struct_function)(py).unwrap();
 
         py_assert!(py, f, "f.__name__ == 'struct'");
     });
@@ -29,14 +33,14 @@ fn test_rust_keyword_name() {
 
 #[pyfunction(signature = (arg = true))]
 fn optional_bool(arg: Option<bool>) -> String {
-    format!("{:?}", arg)
+    format!("{arg:?}")
 }
 
 #[test]
 fn test_optional_bool() {
     // Regression test for issue #932
-    Python::with_gil(|py| {
-        let f = wrap_pyfunction_bound!(optional_bool)(py).unwrap();
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(optional_bool)(py).unwrap();
 
         py_assert!(py, f, "f() == 'Some(true)'");
         py_assert!(py, f, "f(True) == 'Some(true)'");
@@ -45,6 +49,45 @@ fn test_optional_bool() {
     });
 }
 
+#[pyfunction]
+#[pyo3(signature=(arg))]
+fn required_optional_str(arg: Option<&str>) -> &str {
+    arg.unwrap_or("")
+}
+
+#[test]
+fn test_optional_str() {
+    // Regression test for issue #4965
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(required_optional_str)(py).unwrap();
+
+        py_assert!(py, f, "f('') == ''");
+        py_assert!(py, f, "f('foo') == 'foo'");
+        py_assert!(py, f, "f(None) == ''");
+    });
+}
+
+#[pyclass]
+struct MyClass();
+
+#[pyfunction]
+#[pyo3(signature=(arg))]
+fn required_optional_class(arg: Option<&MyClass>) {
+    let _ = arg;
+}
+
+#[test]
+fn test_required_optional_class() {
+    // Regression test for issue #4965
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(required_optional_class)(py).unwrap();
+        let val = Bound::new(py, MyClass()).unwrap();
+
+        py_assert!(py, f val, "f(val) is None");
+        py_assert!(py, f, "f(None) is None");
+    });
+}
+
 #[cfg(not(Py_LIMITED_API))]
 #[pyfunction]
 fn buffer_inplace_add(py: Python<'_>, x: PyBuffer<i32>, y: PyBuffer<i32>) {
@@ -59,8 +102,8 @@ fn buffer_inplace_add(py: Python<'_>, x: PyBuffer<i32>, y: PyBuffer<i32>) {
 #[cfg(not(Py_LIMITED_API))]
 #[test]
 fn test_buffer_add() {
-    Python::with_gil(|py| {
-        let f = wrap_pyfunction_bound!(buffer_inplace_add)(py).unwrap();
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(buffer_inplace_add)(py).unwrap();
 
         py_expect_exception!(
             py,
@@ -103,9 +146,9 @@ fn function_with_pycfunction_arg<'py>(
 
 #[test]
 fn test_functions_with_function_args() {
-    Python::with_gil(|py| {
-        let py_cfunc_arg = wrap_pyfunction_bound!(function_with_pycfunction_arg)(py).unwrap();
-        let bool_to_string = wrap_pyfunction_bound!(optional_bool)(py).unwrap();
+    Python::attach(|py| {
+        let py_cfunc_arg = wrap_pyfunction!(function_with_pycfunction_arg)(py).unwrap();
+        let bool_to_string = wrap_pyfunction!(optional_bool)(py).unwrap();
 
         pyo3::py_run!(
             py,
@@ -118,7 +161,7 @@ fn test_functions_with_function_args() {
 
         #[cfg(not(any(Py_LIMITED_API, PyPy)))]
         {
-            let py_func_arg = wrap_pyfunction_bound!(function_with_pyfunction_arg)(py).unwrap();
+            let py_func_arg = wrap_pyfunction!(function_with_pyfunction_arg)(py).unwrap();
 
             pyo3::py_run!(
                 py,
@@ -134,7 +177,7 @@ fn test_functions_with_function_args() {
 
 #[cfg(not(Py_LIMITED_API))]
 fn datetime_to_timestamp(dt: &Bound<'_, PyAny>) -> PyResult<i64> {
-    let dt = dt.downcast::<PyDateTime>()?;
+    let dt = dt.cast::<PyDateTime>()?;
     let ts: f64 = dt.call_method0("timestamp")?.extract()?;
 
     Ok(ts as i64)
@@ -143,7 +186,7 @@ fn datetime_to_timestamp(dt: &Bound<'_, PyAny>) -> PyResult<i64> {
 #[cfg(not(Py_LIMITED_API))]
 #[pyfunction]
 fn function_with_custom_conversion(
-    #[pyo3(from_py_with = "datetime_to_timestamp")] timestamp: i64,
+    #[pyo3(from_py_with = datetime_to_timestamp)] timestamp: i64,
 ) -> i64 {
     timestamp
 }
@@ -151,8 +194,8 @@ fn function_with_custom_conversion(
 #[cfg(not(Py_LIMITED_API))]
 #[test]
 fn test_function_with_custom_conversion() {
-    Python::with_gil(|py| {
-        let custom_conv_func = wrap_pyfunction_bound!(function_with_custom_conversion)(py).unwrap();
+    Python::attach(|py| {
+        let custom_conv_func = wrap_pyfunction!(function_with_custom_conversion)(py).unwrap();
 
         pyo3::py_run!(
             py,
@@ -170,15 +213,15 @@ fn test_function_with_custom_conversion() {
 #[cfg(not(Py_LIMITED_API))]
 #[test]
 fn test_function_with_custom_conversion_error() {
-    Python::with_gil(|py| {
-        let custom_conv_func = wrap_pyfunction_bound!(function_with_custom_conversion)(py).unwrap();
+    Python::attach(|py| {
+        let custom_conv_func = wrap_pyfunction!(function_with_custom_conversion)(py).unwrap();
 
         py_expect_exception!(
             py,
             custom_conv_func,
             "custom_conv_func(['a'])",
             PyTypeError,
-            "argument 'timestamp': 'list' object cannot be converted to 'PyDateTime'"
+            "argument 'timestamp': 'list' object cannot be cast as 'datetime'"
         );
     });
 }
@@ -196,25 +239,25 @@ fn test_from_py_with_defaults() {
     // issue 2280 combination of from_py_with and Option<T> did not compile
     #[pyfunction]
     #[pyo3(signature = (int=None))]
-    fn from_py_with_option(#[pyo3(from_py_with = "optional_int")] int: Option<i32>) -> i32 {
+    fn from_py_with_option(#[pyo3(from_py_with = optional_int)] int: Option<i32>) -> i32 {
         int.unwrap_or(0)
     }
 
     #[pyfunction(signature = (len=0))]
     fn from_py_with_default(
-        #[pyo3(from_py_with = "<Bound<'_, _> as PyAnyMethods>::len")] len: usize,
+        #[pyo3(from_py_with = <Bound<'_, _> as PyAnyMethods>::len)] len: usize,
     ) -> usize {
         len
     }
 
-    Python::with_gil(|py| {
-        let f = wrap_pyfunction_bound!(from_py_with_option)(py).unwrap();
+    Python::attach(|py| {
+        let f = wrap_pyfunction!(from_py_with_option)(py).unwrap();
 
         assert_eq!(f.call0().unwrap().extract::<i32>().unwrap(), 0);
         assert_eq!(f.call1((123,)).unwrap().extract::<i32>().unwrap(), 123);
         assert_eq!(f.call1((999,)).unwrap().extract::<i32>().unwrap(), 999);
 
-        let f2 = wrap_pyfunction_bound!(from_py_with_default)(py).unwrap();
+        let f2 = wrap_pyfunction!(from_py_with_default)(py).unwrap();
 
         assert_eq!(f2.call0().unwrap().extract::<usize>().unwrap(), 0);
         assert_eq!(f2.call1(("123",)).unwrap().extract::<usize>().unwrap(), 3);
@@ -238,29 +281,26 @@ fn conversion_error(
     option_arg: Option<i64>,
     struct_arg: Option<ValueClass>,
 ) {
-    println!(
-        "{:?} {:?} {:?} {:?} {:?}",
-        str_arg, int_arg, tuple_arg, option_arg, struct_arg
-    );
+    println!("{str_arg:?} {int_arg:?} {tuple_arg:?} {option_arg:?} {struct_arg:?}");
 }
 
 #[test]
 fn test_conversion_error() {
-    Python::with_gil(|py| {
-        let conversion_error = wrap_pyfunction_bound!(conversion_error)(py).unwrap();
+    Python::attach(|py| {
+        let conversion_error = wrap_pyfunction!(conversion_error)(py).unwrap();
         py_expect_exception!(
             py,
             conversion_error,
             "conversion_error(None, None, None, None, None)",
             PyTypeError,
-            "argument 'str_arg': 'NoneType' object cannot be converted to 'PyString'"
+            "argument 'str_arg': 'NoneType' object cannot be cast as 'str'"
         );
         py_expect_exception!(
             py,
             conversion_error,
             "conversion_error(100, None, None, None, None)",
             PyTypeError,
-            "argument 'str_arg': 'int' object cannot be converted to 'PyString'"
+            "argument 'str_arg': 'int' object cannot be cast as 'str'"
         );
         py_expect_exception!(
             py,
@@ -274,7 +314,7 @@ fn test_conversion_error() {
             conversion_error,
             "conversion_error('string1', -100, 'string2', None, None)",
             PyTypeError,
-            "argument 'tuple_arg': 'str' object cannot be converted to 'PyTuple'"
+            "argument 'tuple_arg': 'str' object cannot be cast as 'tuple'"
         );
         py_expect_exception!(
             py,
@@ -334,15 +374,15 @@ fn extract_traceback(py: Python<'_>, mut error: PyErr) -> String {
 fn test_pycfunction_new() {
     use pyo3::ffi;
 
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         unsafe extern "C" fn c_fn(
             _self: *mut ffi::PyObject,
             _args: *mut ffi::PyObject,
         ) -> *mut ffi::PyObject {
-            ffi::PyLong_FromLong(4200)
+            unsafe { ffi::PyLong_FromLong(4200) }
         }
 
-        let py_fn = PyCFunction::new_bound(
+        let py_fn = PyCFunction::new(
             py,
             c_fn,
             c_str!("py_fn"),
@@ -363,10 +403,10 @@ fn test_pycfunction_new() {
 #[test]
 fn test_pycfunction_new_with_keywords() {
     use pyo3::ffi;
-    use std::os::raw::c_long;
+    use std::ffi::c_long;
     use std::ptr;
 
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         unsafe extern "C" fn c_fn(
             _self: *mut ffi::PyObject,
             args: *mut ffi::PyObject,
@@ -389,27 +429,29 @@ fn test_pycfunction_new_with_keywords() {
                 ptr::null_mut(),
             ];
 
-            ffi::PyArg_ParseTupleAndKeywords(
-                args,
-                kwds,
-                c_str!("l|l").as_ptr(),
-                #[cfg(Py_3_13)]
-                args_names.as_ptr(),
-                #[cfg(not(Py_3_13))]
-                args_names.as_mut_ptr(),
-                &mut foo,
-                &mut bar,
-            );
+            unsafe {
+                ffi::PyArg_ParseTupleAndKeywords(
+                    args,
+                    kwds,
+                    c_str!("l|l").as_ptr(),
+                    #[cfg(Py_3_13)]
+                    args_names.as_ptr(),
+                    #[cfg(not(Py_3_13))]
+                    args_names.as_mut_ptr(),
+                    &mut foo,
+                    &mut bar,
+                )
+            };
 
             #[cfg(not(Py_3_13))]
-            drop(std::ffi::CString::from_raw(args_names[0]));
+            drop(unsafe { std::ffi::CString::from_raw(args_names[0]) });
             #[cfg(not(Py_3_13))]
-            drop(std::ffi::CString::from_raw(args_names[1]));
+            drop(unsafe { std::ffi::CString::from_raw(args_names[1]) });
 
-            ffi::PyLong_FromLong(foo * bar)
+            unsafe { ffi::PyLong_FromLong(foo * bar) }
         }
 
-        let py_fn = PyCFunction::new_with_keywords_bound(
+        let py_fn = PyCFunction::new_with_keywords(
             py,
             c_fn,
             c_str!("py_fn"),
@@ -430,36 +472,32 @@ fn test_pycfunction_new_with_keywords() {
 
 #[test]
 fn test_closure() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let f = |args: &Bound<'_, types::PyTuple>,
                  _kwargs: Option<&Bound<'_, types::PyDict>>|
          -> PyResult<_> {
-            Python::with_gil(|py| {
-                let res: Vec<_> = args
+            Python::attach(|py| {
+                let res: PyResult<Vec<_>> = args
                     .iter()
                     .map(|elem| {
                         if let Ok(i) = elem.extract::<i64>() {
-                            (i + 1).into_py(py)
+                            Ok((i + 1).into_pyobject(py)?.into_any().unbind())
                         } else if let Ok(f) = elem.extract::<f64>() {
-                            (2. * f).into_py(py)
+                            Ok((2. * f).into_pyobject(py)?.into_any().unbind())
                         } else if let Ok(mut s) = elem.extract::<String>() {
                             s.push_str("-py");
-                            s.into_py(py)
+                            Ok(s.into_pyobject(py)?.into_any().unbind())
                         } else {
-                            panic!("unexpected argument type for {:?}", elem)
+                            panic!("unexpected argument type for {elem:?}")
                         }
                     })
                     .collect();
-                Ok(res)
+                res
             })
         };
-        let closure_py = PyCFunction::new_closure_bound(
-            py,
-            Some(c_str!("test_fn")),
-            Some(c_str!("test_fn doc")),
-            f,
-        )
-        .unwrap();
+        let closure_py =
+            PyCFunction::new_closure(py, Some(c_str!("test_fn")), Some(c_str!("test_fn doc")), f)
+                .unwrap();
 
         py_assert!(py, closure_py, "closure_py(42) == [43]");
         py_assert!(py, closure_py, "closure_py.__name__ == 'test_fn'");
@@ -474,7 +512,7 @@ fn test_closure() {
 
 #[test]
 fn test_closure_counter() {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let counter = std::cell::RefCell::new(0);
         let counter_fn = move |_args: &Bound<'_, types::PyTuple>,
                                _kwargs: Option<&Bound<'_, types::PyDict>>|
@@ -483,7 +521,7 @@ fn test_closure_counter() {
             *counter += 1;
             Ok(*counter)
         };
-        let counter_py = PyCFunction::new_closure_bound(py, None, None, counter_fn).unwrap();
+        let counter_py = PyCFunction::new_closure(py, None, None, counter_fn).unwrap();
 
         py_assert!(py, counter_py, "counter_py() == 1");
         py_assert!(py, counter_py, "counter_py() == 2");
@@ -502,16 +540,16 @@ fn use_pyfunction() {
         }
     }
 
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         use function_in_module::foo;
 
         // check imported name can be wrapped
-        let f = wrap_pyfunction_bound!(foo, py).unwrap();
+        let f = wrap_pyfunction!(foo, py).unwrap();
         assert_eq!(f.call1((5,)).unwrap().extract::<i32>().unwrap(), 5);
         assert_eq!(f.call1((42,)).unwrap().extract::<i32>().unwrap(), 42);
 
         // check path import can be wrapped
-        let f2 = wrap_pyfunction_bound!(function_in_module::foo, py).unwrap();
+        let f2 = wrap_pyfunction!(function_in_module::foo, py).unwrap();
         assert_eq!(f2.call1((5,)).unwrap().extract::<i32>().unwrap(), 5);
         assert_eq!(f2.call1((42,)).unwrap().extract::<i32>().unwrap(), 42);
     })
@@ -529,7 +567,7 @@ fn return_value_borrows_from_arguments<'py>(
     key: &'py Key,
     value: &'py Value,
 ) -> HashMap<&'py str, i32> {
-    py.allow_threads(move || {
+    py.detach(move || {
         let mut map = HashMap::new();
         map.insert(key.0.as_str(), value.0);
         map
@@ -538,8 +576,8 @@ fn return_value_borrows_from_arguments<'py>(
 
 #[test]
 fn test_return_value_borrows_from_arguments() {
-    Python::with_gil(|py| {
-        let function = wrap_pyfunction_bound!(return_value_borrows_from_arguments, py).unwrap();
+    Python::attach(|py| {
+        let function = wrap_pyfunction!(return_value_borrows_from_arguments, py).unwrap();
 
         let key = Py::new(py, Key("key".to_owned())).unwrap();
         let value = Py::new(py, Value(42)).unwrap();
@@ -551,6 +589,7 @@ fn test_return_value_borrows_from_arguments() {
 #[test]
 fn test_some_wrap_arguments() {
     // https://github.com/PyO3/pyo3/issues/3460
+    #[allow(unused)]
     const NONE: Option<u8> = None;
     #[pyfunction(signature = (a = 1, b = Some(2), c = None, d = NONE))]
     fn some_wrap_arguments(
@@ -562,8 +601,8 @@ fn test_some_wrap_arguments() {
         [a, b, c, d]
     }
 
-    Python::with_gil(|py| {
-        let function = wrap_pyfunction_bound!(some_wrap_arguments, py).unwrap();
+    Python::attach(|py| {
+        let function = wrap_pyfunction!(some_wrap_arguments, py).unwrap();
         py_assert!(py, function, "function() == [1, 2, None, None]");
     })
 }
@@ -579,8 +618,8 @@ fn test_reference_to_bound_arguments() {
         y.map_or_else(|| Ok(x.clone()), |y| y.add(x))
     }
 
-    Python::with_gil(|py| {
-        let function = wrap_pyfunction_bound!(reference_args, py).unwrap();
+    Python::attach(|py| {
+        let function = wrap_pyfunction!(reference_args, py).unwrap();
         py_assert!(py, function, "function(1) == 1");
         py_assert!(py, function, "function(1, 2) == 3");
     })
@@ -606,9 +645,113 @@ fn test_pyfunction_raw_ident() {
         Ok(())
     }
 
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let m = pyo3::wrap_pymodule!(m)(py);
         py_assert!(py, m, "m.struct()");
         py_assert!(py, m, "m.enum()");
     })
 }
+
+#[cfg(not(Py_LIMITED_API))]
+#[pyclass(extends=PyWarning)]
+pub struct UserDefinedWarning {}
+
+#[cfg(not(Py_LIMITED_API))]
+#[pymethods]
+impl UserDefinedWarning {
+    #[new]
+    #[pyo3(signature = (*_args, **_kwargs))]
+    fn new(_args: Bound<'_, PyAny>, _kwargs: Option<Bound<'_, PyAny>>) -> Self {
+        Self {}
+    }
+}
+
+#[test]
+fn test_pyfunction_warn() {
+    #[pyfunction]
+    #[pyo3(warn(message = "TPW: this function raises warning"))]
+    fn function_with_warning() {}
+
+    py_expect_warning_for_fn!(
+        function_with_warning,
+        f,
+        [("TPW: this function raises warning", PyUserWarning)]
+    );
+
+    #[pyfunction]
+    #[pyo3(warn(message = "TPW: this function raises warning with category", category = PyFutureWarning))]
+    fn function_with_warning_with_category() {}
+
+    py_expect_warning_for_fn!(
+        function_with_warning_with_category,
+        f,
+        [(
+            "TPW: this function raises warning with category",
+            PyFutureWarning
+        )]
+    );
+
+    #[pyfunction]
+    #[pyo3(warn(message = "TPW: custom deprecated category", category = pyo3::exceptions::PyDeprecationWarning))]
+    fn function_with_warning_with_custom_category() {}
+
+    py_expect_warning_for_fn!(
+        function_with_warning_with_custom_category,
+        f,
+        [(
+            "TPW: custom deprecated category",
+            pyo3::exceptions::PyDeprecationWarning
+        )]
+    );
+
+    #[cfg(not(Py_LIMITED_API))]
+    #[pyfunction]
+    #[pyo3(warn(message = "TPW: this function raises user-defined warning", category = UserDefinedWarning))]
+    fn function_with_warning_and_user_defined_category() {}
+
+    #[cfg(not(Py_LIMITED_API))]
+    py_expect_warning_for_fn!(
+        function_with_warning_and_user_defined_category,
+        f,
+        [(
+            "TPW: this function raises user-defined warning",
+            UserDefinedWarning
+        )]
+    );
+}
+
+#[test]
+fn test_pyfunction_multiple_warnings() {
+    #[pyfunction]
+    #[pyo3(warn(message = "TPMW: this function raises warning"))]
+    #[pyo3(warn(message = "TPMW: this function raises FutureWarning", category = PyFutureWarning))]
+    fn function_with_multiple_warnings() {}
+
+    py_expect_warning_for_fn!(
+        function_with_multiple_warnings,
+        f,
+        [
+            ("TPMW: this function raises warning", PyUserWarning),
+            ("TPMW: this function raises FutureWarning", PyFutureWarning)
+        ]
+    );
+
+    #[cfg(not(Py_LIMITED_API))]
+    #[pyfunction]
+    #[pyo3(warn(message = "TPMW: this function raises FutureWarning", category = PyFutureWarning))]
+    #[pyo3(warn(message = "TPMW: this function raises user-defined warning", category = UserDefinedWarning))]
+    fn function_with_multiple_custom_warnings() {}
+
+    #[cfg(not(Py_LIMITED_API))]
+    py_expect_warning_for_fn!(
+        function_with_multiple_custom_warnings,
+        f,
+        [
+            ("TPMW: this function raises FutureWarning", PyFutureWarning),
+            (
+                "TPMW: this function raises user-defined warning",
+                UserDefinedWarning
+            )
+        ]
+    );
+}
